@@ -29,22 +29,37 @@
 #include "aven.h"
 #include "message.h"
 
+#include <stdio.h> // for popen
+
+#include <wx/confbase.h>
+
 BEGIN_EVENT_TABLE(AboutDlg, wxDialog)
 END_EVENT_TABLE()
 
 AboutDlg::AboutDlg(wxWindow* parent) :
-    wxDialog(parent, 500, wxString::Format(msg(/*About %s*/205), "Aven"))
+    wxDialog(parent, 500, wxString::Format(msg(/*About %s*/205), APP_NAME))
 {
     wxBoxSizer* horiz = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* vert = new wxBoxSizer(wxVERTICAL);
 
-    wxStaticText* title = new wxStaticText(this, 502, "Aven "VERSION);
-    wxStaticText* purpose = new wxStaticText(this, 503,
-	    wxString(msg(/*Survey visualisation tool*/209)));
-    wxStaticText* copyright1 = new wxStaticText(this, 504,
-	    wxString::Format(AVEN_COPYRIGHT_MSG, msg(/*&copy;*/0)));
-    wxStaticText* copyright2 = new wxStaticText(this, 505,
-	    wxString::Format(COPYRIGHT_MSG, msg(/*&copy;*/0)));
+    if (!bitmap.Ok()) {
+	bitmap.LoadFile(wxString(msg_cfgpth()) + wxCONFIG_PATH_SEPARATOR +
+			wxString("icons") + wxCONFIG_PATH_SEPARATOR +
+			wxString("aven-about.png"), wxBITMAP_TYPE_PNG);
+    }
+    if (bitmap.Ok()) {
+	wxStaticBitmap* static_bitmap = new wxStaticBitmap(this, 501, bitmap);
+	horiz->Add(static_bitmap, 0 /* horizontally unstretchable */, wxALL,
+		   2 /* border width */);
+    }
+    horiz->Add(vert, 0, wxALL, 2);
+
+    wxString id = wxString(APP_NAME" "VERSION"\n");
+    id += msg(/*Survey visualisation tool*/209);
+    wxStaticText* title = new wxStaticText(this, 502, id);
+    wxStaticText* copyright = new wxStaticText(this, 503,
+	    wxString::Format(AVEN_COPYRIGHT_MSG"\n"COPYRIGHT_MSG,
+			     msg(/*&copy;*/0), msg(/*&copy;*/0)));
 
     wxString licence_str;
     wxString l(msg(/*This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public Licence as published by the Free Software Foundation; either version 2 of the Licence, or (at your option) any later version.*/219));
@@ -71,28 +86,18 @@ AboutDlg::AboutDlg(wxWindow* parent) :
 	l = l.substr(a);
     } while (!l.empty());
 
-    wxStaticText* licence = new wxStaticText(this, 508, licence_str);
+    wxStaticText* licence = new wxStaticText(this, 504, licence_str);
     wxButton* ok = new wxButton(this, wxID_OK, wxGetTranslation("OK"));
     ok->SetDefault();
 
-    wxBitmap& bm = wxGetApp().GetAboutBitmap();
-    if (bm.Ok()) {
-	wxStaticBitmap* bitmap = new wxStaticBitmap(this, 501, bm);
-	horiz->Add(bitmap, 0 /* horizontally unstretchable */, wxALL, 2 /* border width */);
-    }
-    horiz->Add(vert, 0, wxALL, 2);
-
     vert->Add(10, 5, 0, wxTOP, 5);
     vert->Add(title, 0, wxLEFT | wxRIGHT, 20);
     vert->Add(10, 5, 0, wxTOP, 5);
-    vert->Add(purpose, 0, wxLEFT | wxRIGHT, 20);
+
+    vert->Add(copyright, 0, wxLEFT | wxRIGHT, 20);
     vert->Add(10, 5, 0, wxTOP, 5);
 
-    vert->Add(copyright1, 0, wxLEFT | wxRIGHT, 20);
-    vert->Add(copyright2, 0, wxLEFT | wxRIGHT, 20);
-    vert->Add(10, 5, 0, wxTOP, 15);
-
-    vert->Add(new wxStaticText(this, 506, msg(/*System Information:*/390)),
+    vert->Add(new wxStaticText(this, 505, msg(/*System Information:*/390)),
 	      0, wxLEFT | wxRIGHT, 20);
 
 #ifdef unix
@@ -119,14 +124,12 @@ AboutDlg::AboutDlg(wxWindow* parent) :
     info += wxVERSION_STRING;
     info += '\n';
     int bpp = wxDisplayDepth();
-    wxString s;
-    s.Printf("Display Depth: %d bpp", bpp);
-    info += s;
+    info += wxString::Format("Display Depth: %d bpp", bpp);
     if (wxColourDisplay()) info += " (colour)";
 
     // Use a readonly multiline text edit for the system info so users can
     // easily cut and paste it into an email when reporting bugs.
-    vert->Add(new wxTextCtrl(this, 507, info, wxDefaultPosition,
+    vert->Add(new wxTextCtrl(this, 506, info, wxDefaultPosition,
 			     wxSize(360, 64), wxTE_MULTILINE|wxTE_READONLY),
 	      0, wxLEFT | wxRIGHT, 20);
 
@@ -139,7 +142,7 @@ AboutDlg::AboutDlg(wxWindow* parent) :
     bottom->Add(5, 5, 1);
     bottom->Add(ok, 0, wxRIGHT | wxBOTTOM, 15);
     vert->Add(bottom, 0, wxEXPAND | wxLEFT | wxRIGHT, 0);
-    vert->SetMinSize(0, bm.GetHeight());
+    vert->SetMinSize(0, bitmap.GetHeight());
 
     SetAutoLayout(true);
     SetSizer(horiz);
