@@ -3,8 +3,8 @@
 //
 //  Main class for Aven.
 //
-//  Copyright (C) 2001, Mark R. Shinwell.
-//  Copyright (C) 2002-2003,2004 Olly Betts
+//  Copyright (C) 2001 Mark R. Shinwell.
+//  Copyright (C) 2002,2003,2004 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -36,9 +36,12 @@
 #include <signal.h>
 
 #include <wx/image.h>
-#include <wx/cmdline.h>
-#include <wx/confbase.h>
-#include <wx/gdicmn.h>
+#if wxUSE_DISPLAY // wxDisplay was added in wx 2.5; but it may not be built
+		  // for mingw (because the header seems to be missing).
+#include <wx/display.h>
+#endif
+//#include <wx/confbase.h>
+//#include <wx/gdicmn.h>
 
 IMPLEMENT_APP(Aven)
 
@@ -66,7 +69,7 @@ bool Aven::OnInit()
 	--argc;
 	memmove(argv + 1, argv + 2, argc * sizeof(char *));
     }
-#endif 
+#endif
     msg_init(argv);
 
     const char *lang = msg_lang2 ? msg_lang2 : msg_lang;
@@ -134,21 +137,27 @@ bool Aven::OnInit()
 
     wxImage::AddHandler(new wxPNGHandler);
 
-    m_AboutBitmap.LoadFile(wxString(msg_cfgpth()) + wxCONFIG_PATH_SEPARATOR +
-			   wxString("icons") + wxCONFIG_PATH_SEPARATOR +
-			   wxString("aven-about.png"), wxBITMAP_TYPE_PNG);
-
     // Obtain the screen size.
-    int width;
-    int height;
-    wxDisplaySize(&width, &height);
+    int x, y;
+    int width, height;
+#if wxUSE_DISPLAY // wxDisplay was added in wx 2.5
+    wxRect geom = wxDisplay().GetGeometry();
+    x = geom.x;
+    y = geom.y;
+    width = geom.width;
+    height = geom.height;
+#else
+    wxClientDisplayRect(&x, &y, &width, &height);
+#endif
 
     // Calculate a reasonable size for our window.
-    int our_width = int(width * 0.75);
-    int our_height = int(our_width * 0.75);
+    x += width / 8;
+    y += height / 8;
+    width = width * 3 / 4;
+    height = height * 3 / 4;
 
     // Create the main window.
-    m_Frame = new MainFrm("Aven", wxPoint(50, 50), wxSize(our_width, our_height));
+    m_Frame = new MainFrm(APP_NAME, wxPoint(x, y), wxSize(width, height));
 
     const bool delay = true;
 
@@ -172,7 +181,7 @@ bool Aven::OnInit()
 
 void Aven::ReportError(const wxString& msg)
 {
-    wxMessageBox(msg, "Aven", wxOK | wxCENTRE | wxICON_EXCLAMATION);
+    wxMessageBox(msg, APP_NAME, wxOK | wxCENTRE | wxICON_EXCLAMATION);
 }
 
 // called to report errors by message.c
