@@ -1,6 +1,6 @@
 /* caverot.c
  * Reads in SURVEX .3d image files & allows quick rotation and examination
- * Copyright (C) 1990,1993-2001 Olly Betts
+ * Copyright (C) 1990,1993-2001,2003 Olly Betts
  * Portions Copyright (C) 1993 Wookey
  *
  * This program is free software; you can redistribute it and/or modify
@@ -228,7 +228,7 @@ draw_names(void)
 # define RETSIGTYPE void
 #endif
 
-static CDECL RETSIGTYPE FAR
+static CDECL RETSIGTYPE Far
 handle_sig(int sig)
 {
    sig = sig;
@@ -812,7 +812,7 @@ show_help(void)
 	{0, "Mouse controls:  LEFT : left/right rotates, up/down zooms", FLAG_MOUSE},
 	{0, "               MIDDLE : up/down tilts", FLAG_MOUSE3},
 	{0, "                RIGHT : mouse moves cave", FLAG_MOUSE2},
-	{/*"No mouse detected"*/100, NULL, FLAG_NOMOUSE},
+	{/*No mouse detected*/100, NULL, FLAG_NOMOUSE},
 	{0, NULL, FLAG_ALWAYS},
 	{0, "              PRESS  ANY  KEY  TO  CONTINUE", FLAG_ALWAYS},
 	{0, NULL, 0}
