@@ -1,10 +1,10 @@
 //
 //  gfxcore.cc
 //
-//  Core drawing code for Aven, with both standard 2D and OpenGL functionality.
+//  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2001, Mark R. Shinwell.
-//  Copyright (C) 2001-2003,2004 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -113,7 +113,7 @@ BEGIN_EVENT_TABLE(GfxCore, wxWindow)
 END_EVENT_TABLE()
 
 GfxCore::GfxCore(MainFrm* parent, wxWindow* parent_win) :
-    wxWindow(parent_win, 100, wxDefaultPosition, wxSize(640, 480)),
+    wxWindow(parent_win, 100),
     m_Font(FONT_SIZE, wxSWISS, wxNORMAL, wxNORMAL, FALSE, "Helvetica",
 	   wxFONTENCODING_ISO8859_1),
     m_InitialisePending(false)
@@ -1183,8 +1183,7 @@ void GfxCore::NattyDrawNames()
     memset((void*)m_LabelGrid, 0, buffer_size);
 
     list<LabelInfo*>::const_iterator label = m_Parent->GetLabels();
-
-    while (label != m_Parent->GetLabelsEnd()) {
+    for ( ; label != m_Parent->GetLabelsEnd(); ++label) {
 	Double x = GridXToScreen((*label)->x, (*label)->y, (*label)->z);
 	Double y = GridYToScreen((*label)->x, (*label)->y, (*label)->z)
 	    + CROSS_SIZE - FONT_SIZE;
@@ -1201,7 +1200,7 @@ void GfxCore::NattyDrawNames()
 	bool reject = true;
 
 	if (ix >= 0 && ix < quantised_x && iy >= 0 && iy < quantised_y) {
-	    char *test = &m_LabelGrid[ix + iy * quantised_x];
+	    char * test = &m_LabelGrid[ix + iy * quantised_x];
 	    int len = str.Length() * dv + 1;
 	    reject = (ix + len >= quantised_x);
 	    int i = 0;
@@ -1215,12 +1214,11 @@ void GfxCore::NattyDrawNames()
 		int ymin = (iy >= 2) ? iy - 2 : iy;
 		int ymax = (iy < quantised_y - 2) ? iy + 2 : iy;
 		for (int y0 = ymin; y0 <= ymax; y0++) {
-                    assert((ix + y0 * quantised_x) < (quantised_x * quantised_y));
+		    assert((ix + y0 * quantised_x) < (quantised_x * quantised_y));
 		    memset((void*) &m_LabelGrid[ix + y0 * quantised_x], 1, len);
 		}
 	    }
 	}
-	++label;
     }
 }
 
@@ -1251,12 +1249,12 @@ void GfxCore::DrawDepthbar()
     for (band = 0; band < m_Bands; band++) {
 	Double z = m_Parent->GetZMin() + m_Parent->GetZExtent() * band
 		/ (m_Bands - 1);
+
 	strs[band] = FormatLength(z, false);
+	
 	int x, dummy;
 	m_DrawDC.GetTextExtent(strs[band], &x, &dummy);
-	if (x > size) {
-	    size = x;
-	}
+	if (x > size) size = x;
     }
 
     int x_min = m_XSize - DEPTH_BAR_OFFSET_X - DEPTH_BAR_BLOCK_WIDTH
@@ -1356,16 +1354,17 @@ wxString GfxCore::FormatLength(Double size_snap, bool scalebar)
 
 void GfxCore::DrawScalebar()
 {
-    if (m_Lock == lock_POINT) return;
-
     // Draw the scalebar.
 
-    // Calculate the extent of the survey, in metres across the screen plane.
+    if (m_Lock == lock_POINT) return;
+
+    // Calculate how many metres of survey are currently displayed across the
+    // screen.
     int x_size = m_XSize;
-
     Double across_screen = Double(x_size / m_Params.scale);
-    Double multiplier = 1.0;
 
+    // Convert to imperial measurements if required.
+    Double multiplier = 1.0;
     if (!m_Metric) {
 	across_screen /= METRES_PER_FOOT;
 	multiplier = METRES_PER_FOOT;
@@ -1388,7 +1387,7 @@ void GfxCore::DrawScalebar()
 
     // Actual size of the thing in pixels:
     int size = int(size_snap * m_Params.scale);
-    m_ScaleBar.width = (int) size; //FIXME
+    m_ScaleBar.width = size;
 
     // Draw it...
     //--FIXME: improve this
@@ -1676,7 +1675,6 @@ void GfxCore::OnMouseMove(wxMouseEvent& event)
 	int y = -(event.GetY() - m_YCentre);
 	Matrix4 inverse_rotation = m_Params.rotation.asInverseMatrix();
 
-	//--TODO: GL version
 	Double cx = Double(inverse_rotation.get(0, 0)*x + inverse_rotation.get(0, 2)*y);
 	Double cy = Double(inverse_rotation.get(1, 0)*x + inverse_rotation.get(1, 2)*y);
 
@@ -1714,7 +1712,7 @@ void GfxCore::OnMouseMove(wxMouseEvent& event)
 	    }
 	    if (m_LastDrag == drag_COMPASS) {
 		// drag in heading indicator
-		double angle = atan2((double)dx0, (double)dy) - M_PI;
+		double angle = atan2(double(dx0), double(dy)) - M_PI;
 		if (dx0 * dx0 + dy * dy <= radius * radius) {
 		    TurnCaveTo(angle);
 		    m_MouseOutsideCompass = false;
@@ -1729,7 +1727,7 @@ void GfxCore::OnMouseMove(wxMouseEvent& event)
 	    else if (m_LastDrag == drag_ELEV) {
 		// drag in elevation indicator
 		if (dx1 >= 0 && dx1 * dx1 + dy * dy <= radius * radius) {
-		    TiltCave(atan2((double)dy, (double)dx1) - m_TiltAngle);
+		    TiltCave(atan2(double(dy), double(dx1)) - m_TiltAngle);
 		    m_MouseOutsideElev = false;
 		}
 		else if (dy >= INDICATOR_MARGIN) {
@@ -1748,8 +1746,6 @@ void GfxCore::OnMouseMove(wxMouseEvent& event)
 	    }
 	    else if (m_LastDrag == drag_SCALE) {
 		if (point.x >= 0 && point.x <= m_XSize) {
-		    //--FIXME: GL fix needed
-
 		    Double size_snap = Double(m_ScaleBar.width) / m_Params.scale;
 		    int dx = point.x - m_DragLast.x;
 
@@ -1766,7 +1762,6 @@ void GfxCore::OnMouseMove(wxMouseEvent& event)
 	    HandleTilt(point);
 	}
 	else if (m_DraggingRight) {
-	  //FIXME: this needs sorting for GL
 	    if ((m_LastDrag == drag_NONE &&
 		 point.x >= m_ScaleBar.offset_x &&
 		 point.x <= m_ScaleBar.offset_x + m_ScaleBar.width &&
@@ -1800,7 +1795,7 @@ void GfxCore::OnSize(wxSizeEvent& event)
 
     wxSize size = event.GetSize();
 
-    if (size.GetWidth() < 0 || size.GetHeight() < 0) {
+    if (size.GetWidth() <= 0 || size.GetHeight() <= 0) {
 	// Before things are fully initialised, we sometimes get a bogus
 	// resize message...
 	return;
@@ -2169,7 +2164,7 @@ void GfxCore::OnPlan()
 	    TiltCave(M_PI_2 - m_TiltAngle);
 	    m_SwitchingTo = 0;
 	    ForceRefresh();
-    } 
+    }
 }
 
 void GfxCore::OnPlanUpdate(wxUpdateUIEvent& cmd)
@@ -2222,7 +2217,6 @@ void GfxCore::OnZoomIn(bool accel)
 {
     // Increase the scale.
 
-    //--GL fixes needed
     SetScale(m_Params.scale * (accel ? 1.1236 : 1.06));
     ForceRefresh();
 }
@@ -2455,10 +2449,6 @@ void GfxCore::OnToggleDegreesUpdate(wxUpdateUIEvent& cmd)
     cmd.Check(m_Degrees);
 }
 
-//
-//  OpenGL-specific methods
-//
-
 void GfxCore::CentreOn(Double x, Double y, Double z)
 {
     m_Params.translation.x = -x;
@@ -2732,8 +2722,8 @@ GfxCore::OnPrint(const wxString &filename, const wxString &title,
 }
 
 bool
-GfxCore::OnExport(const wxString &filename)
+GfxCore::OnExport(const wxString &filename, const wxString &title)
 {
-    return Export(filename, m_Parent,
+    return Export(filename, title, m_Parent,
 	   m_PanAngle, m_TiltAngle, m_Names, m_Crosses, m_Legs, m_Surface);
 }
