@@ -4,7 +4,7 @@
 //  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2001, Mark R. Shinwell.
-//  Copyright (C) 2001-2002,2004 Olly Betts
+//  Copyright (C) 2001-2002,2004,2005 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -385,7 +385,7 @@ public:
 
     void OnPrint(const wxString &filename, const wxString &title,
 		 const wxString &datestamp);
-    bool OnExport(const wxString &filename);
+    bool OnExport(const wxString &filename, const wxString &title);
 
 private:
     DECLARE_EVENT_TABLE()
