@@ -1,6 +1,6 @@
 /* listpos.c
  * SURVEX Cave surveying software: stuff to do with stn position output
- * Copyright (C) 1991-2001 Olly Betts
+ * Copyright (C) 1991-2002 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #define PRINT_STN_POS_LIST 1
 #define NODESTAT 1
 
+#include <limits.h>
+
 #include "cavern.h"
 #include "datain.h"
 #include "debug.h"
@@ -71,16 +73,22 @@ node_stat(prefix *p)
 {
    if (!p->pos) {
       if (!TSTBIT(p->sflags, SFLAGS_SURVEY)) {
-	 /* p doesn't have an associated position, but isn't a survey.
-	  * This means it was referred to in "*entrance" and/or "*export"
-	  * but not elsewhere.
-	  */
-	  warning(/*Station `%s' referred to by *entrance or *export but never used*/190,
-		  sprint_prefix(p));
+	 /* Could do away with the SFLAGS_SURVEY check and check
+	  * p->min_export instead of p->max_export I think ... */
+	 if (TSTBIT(p->sflags, SFLAGS_ENTRANCE) || p->max_export) {
+	     /* p is a station which was referred to in "*entrance" and/or
+	      * "*export" but not elsewhere (otherwise it'd have a position).
+	      * p could also be a survey (SFLAGS_SURVEY) or be mentioned as
+	      * a station, but only in a line of data which was rejected
+	      * because of an error.
+	      */
+	     warning(/*Station `%s' referred to by *entrance or *export but never used*/190,
+		     sprint_prefix(p));
+	 }
       }
    } else {
       int order;
-      ASSERT(pfx_fixed(p));
+      SVX_ASSERT(pfx_fixed(p));
 
       order = p->shape;
 
@@ -104,12 +112,13 @@ node_stat(prefix *p)
 	 printf("L min %d max %d pfx %s\n",
 		p->min_export, p->max_export, sprint_prefix(p));
 #endif
-	 if (p->min_export > 1 || (p->min_export == 0 && p->max_export)) {
+	 if ((p->min_export > 1 && p->min_export != USHRT_MAX) ||
+	     (p->min_export == 0 && p->max_export)) {
 	    char *s;
 	    const char *filename_store = file.filename;
 	    unsigned int line_store = file.line;
 	    prefix *where = p->up;
-	    ASSERT(where);
+	    SVX_ASSERT(where);
 	    s = osstrdup(sprint_prefix(where));
 	    /* Report better when station called 2.1 for example */
 	    while (!where->filename && where->up) where = where->up;
