@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2002 Mark R. Shinwell
-//  Copyright (C) 2001-2003,2004 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005 Olly Betts
 //  Copyright (C) 2004 Philip Underwood
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -65,10 +65,10 @@ class AvenSplitterWindow : public wxSplitterWindow {
 	}
 
 	void OnSplitterDClick(wxSplitterEvent &e) {
-#if wxMAJOR_VERSION > 2 || (wxMAJOR_VERSION == 2 && wxMINOR_VERSION >= 3)
+#if wxCHECK_VERSION(2,3,0)
 	    e.Veto();
 #endif
-#if defined(__UNIX__) && wxMAJOR_VERSION == 2 && wxMINOR_VERSION == 3 && wxRELEASE_NUMBER <= 4
+#if defined(__UNIX__) && !wxCHECK_VERSION(2,3,5)
 	    parent->m_SashPosition = GetSashPosition(); // save width of panel
 	    // Calling Unsplit from OnSplitterDClick() doesn't work in debian
 	    // wxGtk 2.3.3.2 (which calls itself 2.3.4) - it does work from CVS
@@ -79,7 +79,7 @@ class AvenSplitterWindow : public wxSplitterWindow {
 	    parent->ToggleSidePanel();
 #endif
 	}
-	
+
     private:
 	DECLARE_EVENT_TABLE()
 };
@@ -440,11 +440,6 @@ void MainFrm::CreateToolBar()
 		     -1, -1, NULL, "Show surface surveys");
     toolbar->AddSeparator();
 
-#if 0 // FIXME: maybe...
-    m_FindBox = new wxTextCtrl(toolbar, -1, "");
-    toolbar->AddControl(m_FindBox);
-#endif
-
     toolbar->Realize();
 }
 
@@ -630,9 +625,8 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		    info->isLine = false;
 		}
 
-		// Store this point in the list.
+		// Add this point to the list.
 		points.push_back(info);
-
 		break;
 	    }
 
@@ -757,7 +751,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 #endif
 
     // Update window title.
-    SetTitle(wxString("Aven - [") + file + wxString("]"));
+    SetTitle(wxString(APP_NAME" - [") + file + wxString("]"));
     m_File = file;
 
     return true;
@@ -926,9 +920,9 @@ void MainFrm::CentreDataset(Double xmin, Double ymin, Double zmin)
 	}
     }
 
-    list<LabelInfo*>::iterator pos = m_Labels.begin();
-    while (pos != m_Labels.end()) {
-	LabelInfo* label = *pos++;
+    list<LabelInfo*>::iterator lpos = m_Labels.begin();
+    while (lpos != m_Labels.end()) {
+	LabelInfo* label = *lpos++;
 	label->x -= xoff;
 	label->y -= yoff;
 	label->z -= zoff;
@@ -1090,7 +1084,7 @@ void MainFrm::OpenFile(const wxString& file, wxString survey, bool delay)
 
 #undef FILEDIALOG_MULTIGLOBS
 // MS Windows supports "*.abc;*.def" natively; wxGtk supports them as of 2.3
-#if defined(_WIN32) || wxMAJOR_VERSION > 2 || (wxMAJOR_VERSION == 2 && wxMINOR_VERSION >= 3)
+#if defined(_WIN32) || wxCHECK_VERSION(2,3,0)
 # define FILEDIALOG_MULTIGLOBS
 #endif
 
@@ -1100,13 +1094,12 @@ void MainFrm::OnOpen(wxCommandEvent&)
     wxFileDialog dlg (this, wxString(msg(/*Select a 3d file to view*/206)), "", "",
 		      "*.3d", wxOPEN);
 #else
-    wxFileDialog dlg(this, wxString(msg(/*Select a 3d file to view*/206)),
-		     "", "",
+    wxFileDialog dlg(this, wxString(msg(/*Select a 3d file to view*/206)), "", "",
 		     wxString::Format("%s|*.3d"
 #ifdef FILEDIALOG_MULTIGLOBS
 				      ";*.3D"
 #endif
-#ifdef FILEDIALOG_MULTIGLOBS 
+#ifdef FILEDIALOG_MULTIGLOBS
 				      "|%s|*.plt;*.plf"
 #ifndef _WIN32
 				      ";*.PLT;*.PLF"
@@ -1114,7 +1107,7 @@ void MainFrm::OnOpen(wxCommandEvent&)
 #else
 				      "|%s|*.pl?" // not ideal...
 #endif
-			      	      "|%s|*.xyz"
+				      "|%s|*.xyz"
 #ifdef FILEDIALOG_MULTIGLOBS
 #ifndef _WIN32
 				      ";*.XYZ"
@@ -1122,9 +1115,8 @@ void MainFrm::OnOpen(wxCommandEvent&)
 #endif
 				      "|%s|%s",
 				      msg(/*Survex 3d files*/207),
-				      /* FIXME TRANSLATE */
-				      "Compass PLT files",
-				      "CMAP XYZ files",
+				      msg(/*Compass PLT files*/324),
+				      msg(/*CMAP XYZ files*/325),
 				      msg(/*All files*/208),
 				      wxFileSelectorDefaultWildcardStr),
 		     wxOPEN);
@@ -1155,12 +1147,12 @@ void MainFrm::OnExport(wxCommandEvent&)
     char *baseleaf = baseleaf_from_fnm(m_File.c_str());
     wxFileDialog dlg(this, wxString("Export as:"), "",
 		     wxString(baseleaf),
-		     "DXF files|*.dxf|SVG files|*.svg|Sketch files|*.sk|Compass PLT for use with Carto|*.plt",
+		     "DXF files|*.dxf|SVG files|*.svg|Sketch files|*.sk|EPS files|*.eps|Compass PLT for use with Carto|*.plt",
 		     wxSAVE|wxOVERWRITE_PROMPT);
     free(baseleaf);
     if (dlg.ShowModal() == wxID_OK) {
 	wxString fnm = dlg.GetPath();
-	if (!m_Gfx->OnExport(fnm)) {
+	if (!m_Gfx->OnExport(fnm, m_Title)) {
 	    wxGetApp().ReportError(wxString::Format("Couldn't write file `%s'", fnm.c_str()));
 	}
     }
@@ -1406,7 +1398,7 @@ void MainFrm::OnFind(wxCommandEvent&)
     if (!true /* !substring */) {
 	// FIXME "0u" required to avoid compilation error with g++-3.0
 	if (pattern.empty() || pattern[0u] != '^') pattern = '^' + pattern;
-	if (pattern[pattern.size() - 1] != '$') pattern = pattern + '$';
+	if (pattern[pattern.size() - 1] != '$') pattern += '$';
     }
 
     wxRegEx regex;
