@@ -33,6 +33,10 @@
 #include "datain.h" /* for compile_error */
 #include "validate.h" /* for compile_error */
 
+#ifdef CHASM3DX
+#include <stddef.h> /* for offsetof */
+#endif
+
 #define THRESHOLD (REAL_EPSILON * 1000) /* 100 was too small */
 
 node *stn_iter = NULL; /* for FOR_EACH_STN */
@@ -158,9 +162,9 @@ static void check_d(/*const*/ delta *d) {
 /* insert at head of double-linked list */
 void
 add_stn_to_list(node **list, node *stn) {
-   ASSERT(list);
-   ASSERT(stn);
-   ASSERT(stn_iter != stn); /* if it does, we're still on a list... */
+   SVX_ASSERT(list);
+   SVX_ASSERT(stn);
+   SVX_ASSERT(stn_iter != stn); /* if it does, we're still on a list... */
 #if 0
    printf("add_stn_to_list(%p, [%p] ", list, stn);
    if (stn->name) print_prefix(stn->name);
@@ -175,8 +179,8 @@ add_stn_to_list(node **list, node *stn) {
 /* remove from double-linked list */
 void
 remove_stn_from_list(node **list, node *stn) {
-   ASSERT(list);
-   ASSERT(stn);
+   SVX_ASSERT(list);
+   SVX_ASSERT(stn);
 #if 0
    printf("remove_stn_from_list(%p, [%p] ", list, stn);
    if (stn->name) print_prefix(stn->name);
@@ -188,7 +192,7 @@ remove_stn_from_list(node **list, node *stn) {
 	node *stn_to_remove_is_in_list = *list;
 	validate();
 	while (stn_to_remove_is_in_list != stn) {
-	   ASSERT(stn_to_remove_is_in_list);
+	   SVX_ASSERT(stn_to_remove_is_in_list);
 	   stn_to_remove_is_in_list = stn_to_remove_is_in_list->next;
 	}
      }
@@ -218,7 +222,7 @@ copy_link(linkfor *leg)
       for (d = 2; d >= 0; d--) legOut->d[d] = leg->d[d];
    } else {
       leg = reverse_leg(leg);
-      ASSERT(data_here(leg));
+      SVX_ASSERT(data_here(leg));
       for (d = 2; d >= 0; d--) legOut->d[d] = -leg->d[d];
    }
 #if 1
@@ -247,7 +251,7 @@ addto_link(linkfor *leg, const linkfor *leg2)
       adddd(&leg->d, &leg->d, &((linkfor *)leg2)->d);
    } else {
       leg2 = reverse_leg(leg2);
-      ASSERT(data_here(leg2));
+      SVX_ASSERT(data_here(leg2));
       subdd(&leg->d, &leg->d, &((linkfor *)leg2)->d);
    }
    addss(&leg->v, &leg->v, &((linkfor *)leg2)->v);
@@ -268,7 +272,7 @@ addleg_(node *fr, node *to,
    int shape;
    /* we have been asked to add a leg with the same node at both ends
     * - this should be trapped by the caller */
-   ASSERT(fr->name != to->name);
+   SVX_ASSERT(fr->name != to->name);
 
    leg = osnew(linkfor);
    leg2 = (linkfor*)osnew(linkrev);
@@ -369,10 +373,10 @@ static void
 replace_pfx(const prefix *pfx_replace, const prefix *pfx_with)
 {
    pos *pos_replace;
-   ASSERT(pfx_replace);
-   ASSERT(pfx_with);
+   SVX_ASSERT(pfx_replace);
+   SVX_ASSERT(pfx_with);
    pos_replace = pfx_replace->pos;
-   ASSERT(pos_replace != pfx_with->pos);
+   SVX_ASSERT(pos_replace != pfx_with->pos);
 
    replace_pfx_(pfx_replace->stn, NULL, pos_replace, pfx_with->pos);
 
@@ -380,7 +384,7 @@ replace_pfx(const prefix *pfx_replace, const prefix *pfx_with)
    {
       node *stn;
       FOR_EACH_STN(stn, stnlist) {
-	 ASSERT(stn->name->pos != pos_replace);
+	 SVX_ASSERT(stn->name->pos != pos_replace);
       }
    }
 #endif
@@ -527,9 +531,16 @@ StnFromPfx(prefix *name)
    stn = osnew(node);
    stn->name = name;
    if (name->pos == NULL) {
+#ifdef CHASM3DX
+      if (fUseNewFormat) {
+	 name->pos = osnew(pos);
+	 name->pos->id = 0;
+      } else {
+	 /* only allocate the part of the structure we need... */
+	 name->pos = (pos *)osmalloc(offsetof(pos, id));
+      }
+#else
       name->pos = osnew(pos);
-#ifdef NEW3DFORMAT
-      name->pos->id = 0;
 #endif
       unfix(stn);
    }
@@ -543,11 +554,11 @@ StnFromPfx(prefix *name)
 extern void
 fprint_prefix(FILE *fh, const prefix *ptr)
 {
-   ASSERT(ptr);
+   SVX_ASSERT(ptr);
    if (ptr->up != NULL) {
       fprint_prefix(fh, ptr->up);
       if (ptr->up->up != NULL) fputc('.', fh);
-      ASSERT(ptr->ident);
+      SVX_ASSERT(ptr->ident);
       fputs(ptr->ident, fh);
    }
 }
@@ -560,7 +571,7 @@ sprint_prefix_(const prefix *ptr)
 {
    OSSIZE_T len = 1;
    if (ptr->up != NULL) {
-      ASSERT(ptr->ident);
+      SVX_ASSERT(ptr->ident);
       len = sprint_prefix_(ptr->up) + strlen(ptr->ident);
       if (ptr->up->up != NULL) len++;
       if (len > buffer_len) {
@@ -576,7 +587,7 @@ sprint_prefix_(const prefix *ptr)
 extern char *
 sprint_prefix(const prefix *ptr)
 {
-   ASSERT(ptr);
+   SVX_ASSERT(ptr);
    if (!buffer) buffer = osmalloc(buffer_len);
    *buffer = '\0';
    sprint_prefix_(ptr);
@@ -596,8 +607,8 @@ mulvv(var *r, /*const*/ var *a, /*const*/ var *b)
    int i, j, k;
    real tot;
 
-   ASSERT((/*const*/ var *)r != a);
-   ASSERT((/*const*/ var *)r != b);
+   SVX_ASSERT((/*const*/ var *)r != a);
+   SVX_ASSERT((/*const*/ var *)r != b);
 
    check_var(a);
    check_var(b);
@@ -629,8 +640,8 @@ mulss(var *r, /*const*/ svar *a, /*const*/ svar *b)
    real tot;
 
 #if 0
-   ASSERT((/*const*/ var *)r != a);
-   ASSERT((/*const*/ var *)r != b);
+   SVX_ASSERT((/*const*/ var *)r != a);
+   SVX_ASSERT((/*const*/ var *)r != b);
 #endif
 
    check_svar(a);
@@ -663,9 +674,9 @@ smulvs(svar *r, /*const*/ var *a, /*const*/ svar *b)
    real tot;
 
 #if 0
-   ASSERT((/*const*/ var *)r != a);
+   SVX_ASSERT((/*const*/ var *)r != a);
 #endif
-   ASSERT((/*const*/ svar *)r != b);
+   SVX_ASSERT((/*const*/ svar *)r != b);
 
    check_var(a);
    check_svar(b);
@@ -703,7 +714,7 @@ mulvd(delta *r, /*const*/ var *a, /*const*/ delta *b)
    int i, k;
    real tot;
 
-   ASSERT((/*const*/ delta*)r != b);
+   SVX_ASSERT((/*const*/ delta*)r != b);
    check_var(a);
    check_d(b);
 
@@ -729,7 +740,7 @@ mulsd(delta *r, /*const*/ svar *v, /*const*/ delta *b)
    int i, j;
    real tot;
 
-   ASSERT((/*const*/ delta*)r != b);
+   SVX_ASSERT((/*const*/ delta*)r != b);
    check_svar(v);
    check_d(b);
 
@@ -900,7 +911,7 @@ invert_var(var *inv, /*const*/ var *v)
 {
    int i;
    for (i = 0; i < 3; i++) {
-      if ((*v)[i] < THRESHOLD) return 0; /* matrix is singular */
+      if ((*v)[i] == 0.0) return 0; /* matrix is singular */
       (*inv)[i] = 1.0 / (*v)[i];
    }
    return 1;
@@ -912,7 +923,7 @@ invert_var(var *inv, /*const*/ var *v)
    int i, j;
    real det = 0;
 
-   ASSERT((/*const*/ var *)inv != v);
+   SVX_ASSERT((/*const*/ var *)inv != v);
 
    check_var(v);
    for (i = 0; i < 3; i++) {
@@ -920,7 +931,7 @@ invert_var(var *inv, /*const*/ var *v)
 			V((i + 1) % 3, 2) * V((i + 2) % 3, 1));
    }
 
-   if (fabs(det) < THRESHOLD) {
+   if (det == 0.0) {
       /* printf("det=%.20f\n", det); */
       return 0; /* matrix is singular */
    }
@@ -970,11 +981,10 @@ invert_var(var *inv, /*const*/ var *v)
 extern int
 invert_svar(svar *inv, /*const*/ svar *v)
 {
-   int i;
    real det, a, b, c, d, e, f, bcff, efcd, dfbe;
 
 #if 0
-   ASSERT((/*const*/ var *)inv != v);
+   SVX_ASSERT((/*const*/ var *)inv != v);
 #endif
 
    check_svar(v);
@@ -989,7 +999,7 @@ invert_svar(svar *inv, /*const*/ svar *v)
    dfbe = d * f - b * e;
    det = a * bcff + d * efcd + e * dfbe;
 
-   if (fabs(det) < THRESHOLD) {
+   if (det == 0.0) {
       /* printf("det=%.20f\n", det); */
       return 0; /* matrix is singular */
    }
@@ -1010,6 +1020,7 @@ invert_svar(svar *inv, /*const*/ svar *v)
     * well save the cycles and not perform this check.
     */
      { /* check that original * inverse = identity matrix */
+	int i;
 	var p;
 	real D = 0;
 	mulss(&p, v, inv);
