@@ -1,6 +1,6 @@
 /* printps.c & prnthpgl.c */
 /* Device dependent part of Survex Postscript printer/HPGL plotter driver */
-/* Copyright (C) 1993-2002 Olly Betts
+/* Copyright (C) 1993-2003 Olly Betts
  *
  * Postscript font remapping code based on code from a2ps
  * Copyright (C) 1995-1999 Akim Demaille, Miguel Santana
@@ -38,7 +38,7 @@
 #include "message.h"
 #include "prio.h"
 #include "filelist.h"
-#include "debug.h" /* for BUG and ASSERT */
+#include "debug.h" /* for BUG and SVX_ASSERT */
 #include "prcore.h"
 #include "ini.h"
 
@@ -80,6 +80,7 @@ device printer = {
    hpgl_DrawTo,
    hpgl_DrawCross,
    NULL, /* SetFont */
+   NULL, /* SetColour */
    hpgl_WriteString,
    NULL, /* hpgl_DrawCircle */
    hpgl_ShowPage,
@@ -106,6 +107,8 @@ static const char *fontname, *fontname_labels;
 # define PS_TS 9 /* size of alignment 'ticks' on multipage printouts */
 # define PS_CROSS_SIZE 2 /* length of cross arms (in points!) */
 
+static unsigned long colour[6];
+
 static const char *ps_Name(void);
 static int ps_Pre(int pagesToPrint, const char *title);
 static void ps_NewPage(int pg, int pass, int pagesX, int pagesY);
@@ -116,6 +119,7 @@ static void ps_MoveTo(long x, long y);
 static void ps_DrawTo(long x, long y);
 static void ps_DrawCross(long x, long y);
 static void ps_SetFont(int fontcode);
+static void ps_SetColour(int colourcode);
 static void ps_WriteString(const char *s);
 static void ps_DrawCircle(long x, long y, long r);
 static void ps_ShowPage(const char *szPageDetails);
@@ -133,6 +137,7 @@ device printer = {
    ps_DrawTo,
    ps_DrawCross,
    ps_SetFont,
+   ps_SetColour,
    ps_WriteString,
    ps_DrawCircle,
    ps_ShowPage,
@@ -156,6 +161,7 @@ static bool fOriginInCentre = fFalse;
 
 static long x_t, y_t;
 
+/* Check if this line intersects the current page */
 static int
 check_intersection(long x_p, long y_p)
 {
@@ -174,6 +180,9 @@ check_intersection(long x_p, long y_p)
    else if (y_p > ypPageDepth)
       mask_p |= U;
 
+   /* At least one end of the line is on the page */
+   if (mask_p == 0) return 1;
+
    if (x_t < 0)
       mask_t = L;
    else if (x_t > xpPageWidth)
@@ -184,35 +193,58 @@ check_intersection(long x_p, long y_p)
    else if (y_t > ypPageDepth)
       mask_t |= U;
 
-#if 0
-   /* approximation to correct answer */
-   return !(mask_t & mask_p);
-#else
    /* One end of the line is on the page */
-   if (!mask_t || !mask_p) return 1;
+   if (mask_t == 0) return 1;
 
-   /* whole line is above, left, right, or below page */
+   /* See if whole line is above, left, right, or below screen. */
    if (mask_t & mask_p) return 0;
 
-   if (mask_t == 0) mask_t = mask_p;
-   if (mask_t & U) {
-      double v = (double)(y_p - ypPageDepth) / (y_p - y_t);
-      return v >= 0 && v <= 1;
-   }
-   if (mask_t & D) {
-      double v = (double)y_p / (y_p - y_t);
-      return v >= 0 && v <= 1;
-   }
-   if (mask_t & R) {
-      double v = (double)(x_p - xpPageWidth) / (x_p - x_t);
-      return v >= 0 && v <= 1;
-   }
-   ASSERT(mask_t & L);
    {
-      double v = (double)x_p / (x_p - x_t);
-      return v >= 0 && v <= 1;
+      /* Actually need to clip line to screen... */
+      long a = x_p, b = y_p;
+      long c = x_t, d = y_t;
+
+      if (mask_p & U) {
+	 b = ypPageDepth;
+	 a = ((x_p - x_t) * b + x_t * y_p - x_p * y_t) / (y_p - y_t);
+      } else if (mask_p & D) {
+	 b = 0;
+	 a = (x_t * y_p - x_p * y_t) / (y_p - y_t);
+      }
+
+      if (a < 0 || a > xpPageWidth) {
+	 if (!(mask_p & (L|R))) return 0;
+	 if (mask_p & L) {
+	    a = 0;
+	    b = (x_p * y_t - x_t * y_p) / (x_p - x_t);
+	 } else if (mask_p & R) {
+	    a = xpPageWidth;
+	    b = ((y_p - y_t) * a + x_p * y_t - x_t * y_p) / (x_p - x_t);
+	 }
+	 if (b < 0 || b > ypPageDepth) return 0;
+      }
+
+      if (mask_t & U) {
+	 d = ypPageDepth;
+	 c = ((x_p - x_t) * d + x_t * y_p - x_p * y_t) / (y_p - y_t);
+      } else if (mask_t & D) {
+	 d = 0;
+	 c = (x_t * y_p - x_p * y_t) / (y_p - y_t);
+      }
+
+      if (c < 0 || c > xpPageWidth) {
+	 if (!(mask_t & (L|R))) return 0;
+	 if (mask_t & L) {
+	    c = 0;
+	    d = (x_p * y_t - x_t * y_p) / (x_p - x_t);
+	 } else if (mask_t & R) {
+	    c = xpPageWidth;
+	    d = ((y_p - y_t) * c + x_p * y_t - x_t * y_p) / (x_p - x_t);
+	 }
+	 if (d < 0 || d > ypPageDepth) return 0;
+      }
    }
-#endif
+   return 1;
 #undef U
 #undef D
 #undef L
@@ -271,8 +303,7 @@ static char current_font_code = 'F';
 static void
 ps_SetFont(int fontcode)
 {
-   if (fontcode == current_font_code) return;
-
+   char old = current_font_code;
    switch (fontcode) {
     case PR_FONT_DEFAULT:
       current_font_code = 'F';
@@ -283,8 +314,20 @@ ps_SetFont(int fontcode)
     default:
       BUG("unknown font code");
    }
-   prio_putc(current_font_code);
-   prio_putc('\n');
+   if (current_font_code != old) {
+      prio_putc(current_font_code);
+      prio_putc(' ');
+   }
+}
+
+static int current_colourcode = -1;
+static void
+ps_SetColour(int colourcode)
+{
+   if (colourcode != current_colourcode) {
+      current_colourcode = colourcode;
+      prio_printf("C%d ", colourcode);
+   }
 }
 
 static void
@@ -309,7 +352,7 @@ ps_WriteString(const char *s)
    } else {
       int fs = (current_font_code == 'F' ? fontsize : fontsize_labels);
       if ((y_t + 3 * fs / 4 > clip.y_min && y_t - fs / 4 < clip.y_max) ||
-	  (x_t < clip.x_max && x_t + strlen(s) * fs > clip.x_min)) {
+	  (x_t < clip.x_max && x_t + (int)strlen(s) * fs > clip.x_min)) {
 	 fBlankPage = fFalse;
       }
    }
@@ -384,7 +427,9 @@ ps_Pre(int pagesToPrint, const char *title)
    struct passwd *ent;
 #endif
 
-   prio_print("%!PS-Adobe-1.0\n"); /* PS file w/ document structuring */
+   /* PS file w/ EPS document structuring */
+   /* FIXME: prio_print("%!PS-Adobe-3.0 EPSF-3.0\n"); */
+   prio_print("%!PS-Adobe-1.0\n");
 
    prio_print("%%Title: ");
    prio_print(title);
@@ -407,6 +452,7 @@ ps_Pre(int pagesToPrint, const char *title)
       prio_putc('\n');
    }
 
+   /* FIXME: bounding box is too big when border is turned off... */
    prio_printf("%%%%BoundingBox: 0 0 %ld %ld\n",
 	       xpPageWidth + (long)(2.0 * MarginLeft * POINTS_PER_MM),
 	       ypPageDepth + (long)((10.0 + 2.0 * MarginBottom) * POINTS_PER_MM));
@@ -547,6 +593,19 @@ ps_Pre(int pagesToPrint, const char *title)
 	       (int)fontsize_labels);
    prio_print("F\n");
 
+   /* C<digit> changes colour */
+   /* FIXME: read from ini */
+   {
+      size_t i;
+      for (i = 0; i < sizeof(colour) / sizeof(colour[0]); ++i) {
+	 prio_printf("/C%u {stroke %.3f %.3f %.3f setrgbcolor} def\n", i,
+		     (double)(colour[i] & 0xff0000) / 0xff0000,
+		     (double)(colour[i] & 0xff00) / 0xff00,
+		     (double)(colour[i] & 0xff) / 0xff);
+      }
+   }
+   prio_print("C0\n");
+
    /* Postscript definition for drawing a cross */
 #define CS PS_CROSS_SIZE
 #define CS2 (2*PS_CROSS_SIZE)
@@ -620,6 +679,7 @@ ps_NewPage(int pg, int pass, int pagesX, int pagesY)
 	       /*0l,0l, 0l,*/ ypPageDepth,
 	       xpPageWidth, ypPageDepth, xpPageWidth /*,0l*/ );
 #endif
+   current_colourcode = -1;
 #endif
 
 #ifdef HPGL
@@ -685,6 +745,12 @@ ps_Init(FILE **fh_list, const char *pth, const char *out_fnm,
       "line_width",
       "font_labels",
       "font_size_labels",
+      "colour_text",
+      "colour_labels",
+      "colour_frame",
+      "colour_legs",
+      "colour_crosses",
+      "colour_surface_legs",
 #endif
       NULL
    };
@@ -736,6 +802,12 @@ ps_Init(FILE **fh_list, const char *pth, const char *out_fnm,
    if (vals[10]) fontname_labels = vals[10];
    fontsize_labels = fontsize;
    if (vals[11]) fontsize_labels = as_int(vars[11], vals[11], 1, INT_MAX);
+   {
+      size_t i;
+      for (i = 0; i < sizeof(colour) / sizeof(colour[0]); ++i) {
+	 if (vals[12 + i]) colour[i] = as_colour(vars[12 + i], vals[12 + i]);
+      }
+   }
 
    osfree(vals);
 
@@ -867,7 +939,7 @@ hpgl_WriteString(const char *s)
    } else {
 #define CHAR_SIZE (6 * HPGL_UNITS_PER_MM) /* Guesstimate of character size */
       if ((y_t + CHAR_SIZE > clip.y_min && y_t < clip.y_max) ||
-	  (x_t < clip.x_max && x_t + strlen(s) * CHAR_SIZE > clip.x_min)) {
+	  (x_t < clip.x_max && x_t + (int)strlen(s) * CHAR_SIZE > clip.x_min)) {
 	 fBlankPage = fFalse;
       }
    }
